/*	License
 * 		inside EU: EUPL
 * 		outside EU: GPL or EUPL
 *  The program comes AS IS and without any warranty unless required by law
 *  (C) 2012 thecodemonkey
 */

package gui.layout;

import java.awt.Component;
import java.awt.Container;
import java.awt.Dimension;
import java.awt.Insets;
import java.awt.LayoutManager2;
import java.util.HashMap;
import java.util.Map;

public class AnchorLayout implements LayoutManager2 {

	private Map<Component, AnchorLayoutConstraints> constraints;
	private Dimension minSize;
	
	public AnchorLayout() {
		constraints = new HashMap<Component, AnchorLayoutConstraints>();
		minSize = new Dimension(0, 0);
	}
	
	@Override
	public void addLayoutComponent(String arg0, Component arg1) {
		
	}

	@Override
	public void layoutContainer(Container target) {
		synchronized (target.getTreeLock()) {
			int count=target.getComponentCount();
			
			Insets border=target.getInsets();
			int width = target.getWidth() - border.left-border.right;
			int height = target.getHeight() - border.top-border.bottom;
			
			for (int i=0;i<count;i++) {
				Component comp = target.getComponent(i);
				AnchorLayoutConstraints c = constraints.get(comp);
				
				if(c!=null) {
					int x = c.getLeft();
					int y = c.getTop();
					int w = c.getWidth()>0?c.getWidth():comp.getMinimumSize().width;
					int h = c.getHeight()>0?c.getHeight():comp.getMinimumSize().height;
					
					if (c.left() && c.right()) {
						w = width - c.getRight() - c.getLeft();  
					} else if (c.right()) {
						x = width- c.getRight() - w; 
					} else if (c.distrH()) {
						//int pos=(i+1)*100/count;
						x = ( (width/count-w)/2)+(width*i/count);
					} else if (!c.left()){
						x = (width-w)/2;
					} 
					
					if (c.top() && c.bottom()) {
						h = height - c.getBottom() - c.getTop();  
					} else if (c.bottom()) {
						y = height- c.getBottom() - h; 
					
					} else if (c.distrV()){
						int pos=i*100/count;
						y=((height/count-h)/2)+height*pos/100;
					} else if (!c.top()){
						y = (height-h)/2;
					}
					
					comp.setBounds(x+border.left, y+border.top, w, h);
				}
			}

		}
	}

	@Override
	public Dimension minimumLayoutSize(Container parent) {
		return minSize;
	}

	@Override
	public Dimension preferredLayoutSize(Container parent) {
		return parent.getSize();
	}

	@Override
	public void removeLayoutComponent(Component c) {
		constraints.remove(c);
	}

	@Override
	public void addLayoutComponent(Component c, Object constraint) {
		synchronized (c.getTreeLock()) {
			if (constraint instanceof AnchorLayoutConstraints) {
				AnchorLayoutConstraints cons=(AnchorLayoutConstraints) constraint;
//				if (cons.getWidth()>-1)minSize.width=cons.getWidth();
//				if (cons.getHeight()>-1)minSize.height=cons.getHeight();
				constraints.put(c, cons);
			}
		}
	}

	@Override
	public float getLayoutAlignmentX(Container target) {
		return 0.5f;
	}

	@Override
	public float getLayoutAlignmentY(Container target) {
		return 0.5f;
	}

	@Override
	public void invalidateLayout(Container arg0) {
		// TODO Auto-generated method stub

	}

	@Override
	public Dimension maximumLayoutSize(Container target) {
		return target.getSize();
	}

	public static AnchorLayoutConstraints getConstraints(AnchorParam... pos){
		return AnchorLayoutConstraints.get(pos);
	}
	
	public static AnchorLayoutConstraints getConstraintsBySize(int width, int height) {
		return AnchorLayoutConstraints.getBySize(width, height);
	}
	
	public static AnchorLayoutConstraints getConstraintsByBorder(int left,int right,int top,int bottom) {
		return AnchorLayoutConstraints.getByBorder(left, right, top, bottom);
	}
}
