/*	License
 * 		inside EU: EUPL
 * 		outside EU: GPL or EUPL
 *  The program comes AS IS and without any warranty unless required by law
 *  (C) 2012 thecodemonkey
 */

package gui.settings.dialog.page;


import gui.layout.percent2.AnchorPercentLayout;
import gui.resource.ResourceManager;
import gui.settings.dialog.component.SettingsComponent;
import gui.settings.dialog.group.SettingsComponentGroup;

import java.awt.GridLayout;
import java.util.LinkedList;
import java.util.List;

import javax.swing.BorderFactory;
import javax.swing.JComponent;

import core.io.Application;

import settings.Settings;

public abstract class SettingsPage extends AbstractSettingsPage {
	private static final long serialVersionUID = -5365695989495313718L;
	protected SettingsComponentGroup singleComponent;
	protected List<SettingsComponentGroup> components;
		
	
	public SettingsPage(String entry,Application app) {
		super(app.getSettings());
		this.entry=entry;
		createComponentGroups(app.getSettings(),app.getResourceManager());
		setBorder();
	}

	
	/**
	 * equal distribution by default
	 */
	protected void setLayoutMgr(int componentCount) {
		setLayout(new GridLayout(componentCount, 1,0,20));
	}
	
	
	protected void setBorder() {
		setBorder(BorderFactory.createEmptyBorder(20, 20, 20, 20));
	}
	
	protected void addSettingsComponentGroup(SettingsComponentGroup grp) {
		addSettingsComponentGroup(grp, null);
	}
	
	protected void addSettingsComponentGroup(SettingsComponentGroup grp,JComponent parent,Object constraints) {
		addToCompList(grp);
		parent.add(grp,constraints );
	}
	
	protected void addSingleComponent(String borderText, final SettingsComponent c,ResourceManager rm,Object groupConstraints,final Object compConstraints) {
		addSettingsComponentGroup(new SettingsComponentGroup(borderText, rm) {
			
			private static final long serialVersionUID = -6645130313675583881L;

			@Override
			protected void createSettingsComponents(ResourceManager rm) {
				setLayout(new AnchorPercentLayout());
				addSettingsComponent(c,compConstraints);
			}
		},groupConstraints);
	}
	
	private void addToCompList(SettingsComponentGroup grp) {
		if (singleComponent==null&&components==null) {
			singleComponent = grp;
		} else {
			if (components==null) {
				components = new LinkedList<>();
				components.add(singleComponent);
				singleComponent=null;
			} 
			components.add(grp);
		}
	}
	
	protected void addSettingsComponentGroup(SettingsComponentGroup grp,Object constraints) {
		addToCompList(grp);
		add(grp,constraints);
	}
	
	protected abstract void createComponentGroups(Settings settings,ResourceManager rm);
	
	@Override
	public String toString() {
		return entry;
	}

	@Override
	public void load() {
		if (singleComponent!=null) {
			singleComponent.loadSettings(settings);
		} else {
			for (SettingsComponentGroup c:components) {
				c.loadSettings(settings);
			}
		}
	}
	
	@Override
	public void save() {
		if (singleComponent!=null) {
			singleComponent.saveSettings(settings);
		} else {
			for (SettingsComponentGroup c:components) {
				c.saveSettings(settings);
			}
		}
	}


	public String getEntry() {
		return entry;
	}
}
